use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Amazon::API::Object::MapperMWSProduct;
use Test::More;
use Data::Dumper;
use File::Slurp;

sub Main {
    testMapperMWSProduct();
    done_testing();
}

sub testMapperMWSProduct {
    my $o = DE_EPAGES::Amazon::API::Object::MapperMWSProduct->new('abc','def');
    is($o->{'STDURL'},'abc','stdURL namespace URL');
    is($o->{'ANSURL'},'def','ansURL namespace URL');

    LoadXMLToDTO('MWSParentProductResponse.xml');
    LoadXMLToDTO('MWSSubProductResponse.xml');
}

sub LoadXMLToDTO{
    my ($FileName) = @_;
    my $o = DE_EPAGES::Amazon::API::Object::MapperMWSProduct->new();
    is($o->{'STDURL'},'http://mws.amazonservices.com/schema/Products/2011-10-01/default.xsd','stdURL namespace URL');
    is($o->{'ANSURL'},'http://mws.amazonservices.com/schema/Products/2011-10-01','ansURL namespace URL');
    my $XML = read_file($FileName);
    my $DTOList = $o->mapXMLToDTO($XML,'aURL');
    my $f = $DTOList->[0]; # get first Element
    is($f->ASIN,'42','get product ASIN');
    is($f->URL,'aURL/dp/42','get product URL');
    is($f->Binding,'Binding','get Binding');
    is($f->Brand,'Brand','get Brand');
    is($f->Department,'Department','get Department');
    is($f->Color,'Color','get Color');
    is($f->Feature,'Feature1 - Feature2 - Feature3','get Feature');
    is($f->DimensionHeight,'666.1','get DimensionHeight');
    is($f->DimensionLength,'666.2','get DimensionLength');
    is($f->DimensionWidth,'666.3','get DimensionWidth');
    is($f->DimensionWeight,'666.4','get DimensionWeight');
    is($f->DimensionLenghtUnit,'inches','get DimensionHeightUnit');
    is($f->DimensionWeightUnit,'pounds','get DimensionWeightUnit');
    is($f->Label,'Label','get Label');
    is($f->ListPrice,'666.5','get ListPrice');
    is($f->ListPriceCurrency,'EUR','get ListPriceCurrency');
    is($f->Manufacturer,'Manufacturer','get Manufacturer');
    is($f->MaterialType,'MaterialType','get MaterialType');
    is($f->Model,'666','get Model');
    is($f->PackageDimensionsHeight,'666.6','get PackageDimensionsHeight');
    is($f->PackageDimensionsLength,'666.7','get PackageDimensionsLength');
    is($f->PackageDimensionsWidth,'666.8','get PackageDimensionsWidth');
    is($f->PackageDimensionsWeight,'666.9','get PackageDimensionsWeight');
    is($f->PackageDimensionLenghtUnit,'inches','get PackageDimensionLenghtUnit');
    is($f->PackageDimensionWeightUnit,'pounds','get PackageDimensionWeightUnit');
    is($f->PartNumber,'66610','get PartNumber');
    is($f->ProductGroup,'ProductGroup','get ProductGroup');
    is($f->ProductTypeName,'ProductTypeName','get ProductTypeName');
    is($f->Publisher,'Publisher','get Publisher');
    is($f->ReleaseDate,'0001-01-01','get ReleaseDate');
    is($f->Size,'Size','get Size');
    is($f->ImageURL,'URL','get ImageURL');
    is($f->ImageHeight,'66611','get ImageHeight');
    is($f->ImageWidth,'66612','get ImageWidth');
    is($f->Studio,'Studio','get Studio');
    is($f->Title,'Title','get Title');
    if ( $f->VariationParentASIN ){
        is($f->VariationParentASIN,'abcdef','get VariationParentASIN');
        is($f->VariationsProducts, 0 ,'get undefined VariationsProducts');
    }
    else{
        is($f->VariationParentASIN,0,'get undefined VariationParentASIN');
        # ask for Sub product Attribute if you need more test goto testMWSSubProduct.t
        is(ref $f->VariationsProducts,'HASH','get VariationsProducts is a HASH');
        is ($f->VariationsProducts->{'DataSets'}->[0]->{'Data'}[1],'Color2','get a example sub product attribute');
    }
}
RunScript( Sub => \&Main );