use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::AvaTax::API::AvaTax;
use DE_EPAGES::AvaTax::API::DTO::GetTaxRequest;
use DE_EPAGES::AvaTax::API::DTO::BaseAddress;
use DE_EPAGES::AvaTax::API::DTO::Line;
use DE_EPAGES::AvaTax::API::Constants qw ( 
    AVATAX_DETAILLEVEL_Tax
);

use strict;

sub run_tests {
    my $Request = getTaxRequest();
    ## no critic (ProtectPrivateSubs)
    my $Hash = DE_EPAGES::AvaTax::API::AvaTax::_CreateRequestHash($Request);
    is($Hash, '7357cbdd5ec1c24c6ab6acd9852a0ad1','hash compare.');
}

sub getTaxRequest {
    my $Request = DE_EPAGES::AvaTax::API::DTO::GetTaxRequest->new;
    $Request->CompanyCode('EPAGE');
    $Request->OriginCode('Origin');
    $Request->DestinationCode('Dest');
    $Request->DetailLevel(AVATAX_DETAILLEVEL_Tax);
    my $Origin = DE_EPAGES::AvaTax::API::DTO::BaseAddress->new;
    $Origin->AddressCode('Origin');
    $Origin->Line1('Avalara');
    $Origin->Line2('900 Winslow Way');
    $Origin->Line3('Suite 100');
    $Origin->City('Bainbridge Island');
    $Origin->Region('WA');
    $Origin->PostalCode('98110');
    $Origin->Country('USA');
    $Request->pushAddress($Origin);

    my $Dest = DE_EPAGES::AvaTax::API::DTO::BaseAddress->new;
    $Dest->AddressCode('Dest');
    $Dest->Line1('3130 Elliott');
    $Dest->City('Seattle');
    $Dest->Region('WA');
    $Dest->PostalCode('98121');
    $Dest->Country('USA');
    $Request->pushAddress($Dest);

    #product
    my $Line = DE_EPAGES::AvaTax::API::DTO::Line->new;
    $Line->No(1);
    $Line->ItemCode('Item0001');
    # 256 chars. - cut after 255 chars.
    $Line->Description('1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456');
    $Line->Qty(1);
    $Line->Amount(1000);
    $Request->pushLine($Line);

    #Shipping.
    $Line = DE_EPAGES::AvaTax::API::DTO::Line->new;
    $Line->No(2);
    $Line->ItemCode('FR');
    $Line->Qty(1);
    $Line->Amount(10.00);
    $Request->pushLine($Line);

    return $Request;
}


sub Main {
    run_tests;
    done_testing;
}

RunScript( Sub => \&Main );

1;
