use strict;

use Test::Simple tests => 13;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw (
    ConnectTestSite
    DisconnectSite
    GetCurrentDBHandle
    GetTestSite
    RunOnStore
);
use DE_EPAGES::Database::API::DatabaseInstaller;

# EPG-24188
sub Main {
    ConnectTestSite();

    my $dbi = GetCurrentDBHandle();
    my $driverName = $dbi->driverName;
    ok( $dbi, 'GetCurrentDBHandle driverName = ' . $driverName );
    ok( Init( $dbi ), 'Init' );

    my $Index;
    my @aColumns = qw( displayname street zipcode city );
    foreach my $Column ( @aColumns ) {
        $Index = 'i_' . $Column;
        $dbi->createIndex( 'testAddress', $Index, undef, [ $Column ] );
        ok( $dbi->existsIndex( 'testAddress', $Index), "Create index $Index" );
    }

    my $aIndexes = [ 'i_displayname', 'i_zipcode' ];
    DE_EPAGES::Database::API::DatabaseInstaller::dropIndexesIfExist( undef, 'testAddress', $aIndexes );
    foreach my $Index ( @{ $aIndexes } ) {
        ok( !$dbi->existsIndex( 'testAddress', $Index ), "Index $Index already dropped" );
    }
    ok( $dbi->existsIndex( 'testAddress', 'i_street' ), "Index $Index exists" );
    ok( $dbi->existsIndex( 'testAddress', 'i_city' ), "Index $Index exists" );

    $aIndexes = [ 'i_city' ];
    DE_EPAGES::Database::API::DatabaseInstaller::dropIndexesIfExist( undef, 'testAddress', $aIndexes );
    ok( !$dbi->existsIndex( 'testAddress', 'i_city' ), "Index $Index already dropped" );
    ok( $dbi->existsIndex( 'testAddress', 'i_street' ), "Index $Index exists" );

    ok( Cleanup( $dbi ), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    my ($dbi) = @_;

    $dbi->dropTableIfExists( 'testAddress' );

    return 1;
}

sub Init {
    my ($dbi) = @_;

    Cleanup( $dbi );
    $dbi->do(<<END_SQL);
CREATE TABLE testAddress (
    addressid INTEGER PRIMARY KEY,
    displayname VARCHAR(50) NULL,
    street VARCHAR(255) NULL,
    zipcode VARCHAR(25) NULL,
    city VARCHAR(60) NULL
)
END_SQL

    return 1;
}

RunScript( Message => 'long', Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});

1;
