use strict;
use Data::Dumper;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UPS::API::XML::Handler;
use XML::Compile::WSDL11;
use DE_EPAGES::UPS::t::MockSOAPCall;
use DE_EPAGES::UPS::API::XML::Parser::Parser;

use XML::LibXML::Reader;
use Readonly;
use Test::More;
use Test::Exception;

Readonly my $ROOTPATH => '/TestStore';
## no critic (ProhibitLongLines)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            ## no critic (ProhibitPrivateSubroutineUse ProtectPrivateVars RequireInitializationForLocalVars)
            eval {
                my $parser  = DE_EPAGES::UPS::API::XML::Parser::Parser->new();
                { #mock _GetErrorValues
                    local *DE_EPAGES::UPS::API::XML::Parser::Parser::_getErrorValues = sub {return 'VarsCode','VarsDescription';};
                    eval{
                        $parser->disassemble(errorHash());
                    };
                    if ( ExistsError() ) {
                            my $Error = GetError();
                            is ($Error->{'Vars'}->{'Code'},'VarsCode','Error->Vars->Code OK');
                            is ($Error->{'Vars'}->{'Description'},'VarsDescription','Error->Vars-> Description OK');
                    }
                }
                my ($code,$description) = $parser->_getErrorValues(errorHash());
                is($code,'250003','Error Code OK');
                is($description,'Invalid Access License number','Error Description OK');
                {#mock _GetBase64EncodedShippingLabelPicture
                    local *DE_EPAGES::UPS::API::XML::Parser::Parser::_getBase64EncodedShippingLabelPicture = sub {return 'SGFsbG8gV3Vyc3Qh';};
                    is ($parser->_getShippingLabelPicture(successHash()),'Hallo Wurst!',' is ShippingLabelPicture ');
                }
                my $Response = successHash();
                my $PackageResults  = $Response->{'Body'}{'ShipmentResults'}{'PackageResults'}[0];
                my $ShipmentCharges = $Response->{'Body'}{'ShipmentResults'}{'ShipmentCharges'};
                is ($parser->_getBase64EncodedShippingLabelPicture($PackageResults),'abcdefg',' is Base64EncodedShippingLabelPicture ');
                is ($parser->_getTrackingNumber($PackageResults),'123456789abcdef',' is TrackingNumber ');
                is ($parser->_getTotalCharges($ShipmentCharges),'TotalCharges',' is TotalCharges ');
                is ($parser->_getServiceOptionsCharges($ShipmentCharges),'ServiceOptionsCharges',' is ServiceOptionsCharges ');
                is ($parser->_getTransportationCharges($ShipmentCharges),'TransportationCharges',' is TransportationCharges ');
                is ($parser->_getCurrencyCode($ShipmentCharges),'CurrencyCode',' is CurrencyCode ');
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }
            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'importOrder.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

sub errorHash {
    return {
        'Fault' => {
            'detail' => {
                'Errors' => {
                    'ErrorDetail' => [ {
                            'PrimaryErrorCode' => {
                                'Code'        => '250003',
                                'Description' => 'Invalid Access License number'
                            },
                        }
                    ]
                }
            },
        }
    };
}
sub successHash {
    return {
        'Body' => {
            'ShipmentResults' => {
                'PackageResults' => [ {
                    'ShippingLabel' => {
                        'GraphicImage' => 'abcdefg' ,
                    },
                    'TrackingNumber' => '123456789abcdef',
                } ],
                'ShipmentCharges' => {
                    'TotalCharges' => {
                        'CurrencyCode' => 'CurrencyCode',
                        'MonetaryValue' => 'TotalCharges',
                    },
                    'ServiceOptionsCharges' => {
                        'MonetaryValue' => 'ServiceOptionsCharges',
                    },
                    'TransportationCharges' => {
                        'MonetaryValue' => 'TransportationCharges',
                    },
                }
            }
        }
    };
}
1;
